from rs_datasets.movielens import MovieLens
from rs_datasets.msd import MillionSongDataset
from rs_datasets.netflix import Netflix
from rs_datasets.lastfm import Lastfm
from rs_datasets.epinions import Epinions
from rs_datasets.goodreads import Goodreads
from rs_datasets.dating_agency import DatingAgency
from rs_datasets.book_crossing import BookCrossing
from rs_datasets.jester import Jester
from rs_datasets.amazon import Amazon



import os
import tarfile
from os.path import splitext
from tarfile import TarFile
from typing import Union
from zipfile import ZipFile


def extract(archive_name: str, manage_folder: bool = True) -> None:
    """
    Extract `archive_name` and put it inside a folder
    if there are multiple files inside.

    :param archive_name: path to archive
    :param manage_folder: check if there is root folder in archive:
        if there is one, do not create extra folder,
        if there are just files inside, put them into folder.
        If param is set to `False`, extract "as is".
    :return:
    """
    if archive_name.endswith('.zip'):
        archive = ZipFile(archive_name)
    else:
        try:
            archive = tarfile.open(archive_name)
        except Exception:
            raise NotImplementedError(f'Can\'t extract {archive_name}')

    name = os.path.dirname(archive_name)
    if manage_folder and not contains_dir(archive):
        name = remove_extension(archive_name)
        os.mkdir(name)

    archive.extractall(path=name)
    archive.close()


def rm_if_exists(filepath: str) -> None:
    """
    Remove file if it exists, else do nothing.

    :param filepath: path to file
    :return: None
    """
    if os.path.exists(filepath):
        os.remove(filepath)


def contains_dir(archive: Union[ZipFile, TarFile]) -> bool:
    """
    Check if archive contains a root folder or just files.

    :param archive: archive file
    :return: `True` if first element of archive is folder
    """
    if isinstance(archive, ZipFile):
        contents = archive.infolist()
        is_dir = contents[0].is_dir()
    elif isinstance(archive, TarFile):
        contents = archive.getmembers()
        is_dir = contents[0].isdir()
    else:
        raise TypeError(f'Unknown archive type: {type(archive)}')
    return is_dir


def remove_extension(file: str) -> str:
    """
    Get file name without _last_ extension.

    :param file: string
    :return: archive.tar.gz -> archive.tar
    """
    return splitext(file)[0]



from os import mkdir, rename
from os.path import join, exists

import pandas as pd
import numpy as np

from rs_datasets.data_loader import download_dataset
from rs_datasets.generic_dataset import Dataset, safe


class Jester(Dataset):
    def __init__(self, dataset: int = 1, path: str = None):
        """
        :param dataset: version of dataset (1, 3, 4).
        :param path: folder which is used to download dataset to
            if it does not contain dataset files.
            If files are found, load them.
        """
        super().__init__(path)
        if dataset not in {1, 3, 4}:
            raise ValueError('Dataset version must be one of {1, 3, 4}')
        folder = join(self.data_folder, 'jester')
        if not exists(folder):
            self._download(folder)

        if dataset == 1:
            data = []
            for i in [1, 2, 3]:
                ratings = pd.read_excel(join(folder, str(dataset), f'data_{i}.xls'), header=None)
                ratings = self._process(ratings)
                data.append(ratings)
            self.data1 = data[0]
            self.data2 = data[1]
            self.data3 = data[2]
            jokes = pd.read_excel(join(folder, '3', f'jokes.xlsx'), header=None)
            self.jokes = self._fix_jokes(jokes)
        else:
            data = pd.read_excel(join(folder, str(dataset), f'data.xlsx'), header=None)
            self.data = self._process(data)
            jokes = pd.read_excel(join(folder, str(dataset), f'jokes.xlsx'), header=None)
            self.jokes = self._fix_jokes(jokes)

    @staticmethod
    def _process(df):
        df = df.drop(0, axis=1)
        df = df.replace(99, np.nan)
        df = df.astype(pd.SparseDtype('float', np.nan))
        return df

    @staticmethod
    def _fix_jokes(df):
        df.index = range(1, len(df) + 1)
        return df[0]

    @safe
    def _download(self, path):
        self.logger.info('Downloading Jester dataset...')
        mkdir(path)
        base_url = 'http://eigentaste.berkeley.edu/dataset/'

        self.logger.info('Dataset 1...')
        d1 = join(path, '1')
        mkdir(d1)
        download_dataset(
            base_url + 'jester_dataset_1_joke_texts.zip',
            join(d1, 'joke_texts.zip'),
            manage_folder=False
        )
        for i in [1, 2, 3]:
            download_dataset(
                base_url + f'jester_dataset_1_{i}.zip',
                join(d1, f'ratings_{i}.zip'),
                manage_folder=False
            )
            rename(join(d1, f'jester-data-{i}.xls'), join(d1, f'data_{i}.xls'))

        for i in [3, 4]:
            self.logger.info(f'Dataset {i}...')
            d = join(path, str(i))
            mkdir(d)
            download_dataset(
                base_url + f'Dataset{i}JokeSet.zip',
                join(d, 'joke_texts.zip'),
                manage_folder=False
            )
            rename(join(d, f'Dataset{i}JokeSet.xlsx'), join(d, 'jokes.xlsx'))
            download_dataset(
                base_url + f'JesterDataset{i}.zip',
                join(d, 'ratings.zip'),
                manage_folder=False
            )
            if i == 3:
                name = 'FINAL jester 2006-15.xls'
            else:
                name = '[final] April 2015 to Nov 30 2019 - Transformed Jester Data - .xlsx'
            rename(join(d, name), join(d, 'data.xlsx'))



import logging
from os import mkdir, rename
from os.path import  join

from rs_datasets.data_loader.loaders import download_dataset


def download_rekko(path: str = '.'):
    """
    Скачать датасет с rekko chalenge
    https://boosters.pro/championship/rekko_challenge/data
    175MB

    :param path: куда положить
    :return: None
    """
    logging.info('Downloading rekko challenge dataset...')
    archive = 'rekko_challenge_rekko_challenge_2019.zip'
    url = f'https://boosters.pro/api/ch/files/pub/{archive}'
    path = join(path, 'rekko.zip')
    download_dataset(url, path)


def download_citeulike_a(path: str = '.'):
    """
    Скачать CiteULike-a
    https://github.com/js05212/citeulike-a

    :param path: куда положить
    :return: None
    """
    logging.info('Downloading CiteULike-a dataset...')
    url = 'https://github.com/js05212/citeulike-a/archive/master.zip'
    download_dataset(url, join(path, 'citeulike-a.zip'))
    rename(join(path, 'citeulike-a-master'), join(path, 'citeulike-a'))


def download_hetrec(path: str = '.'):
    """
    Скачать HetRec 2011
    https://grouplens.org/datasets/hetrec-2011/

    :param path: куда положить
    :return: None
    """
    logging.info('Downloading HetRec 2011 dataset...')
    folder = join(path, 'hetrec')
    mkdir(folder)
    base_url = 'http://files.grouplens.org/datasets/hetrec2011/'

    download_dataset(
        base_url + 'hetrec2011-delicious-2k.zip',
        join(folder, 'delicious.zip')
    )
    download_dataset(
        base_url + 'hetrec2011-lastfm-2k.zip',
        join(folder, 'lastfm.zip')
    )
    download_dataset(
        base_url + 'hetrec2011-movielens-2k-v2.zip',
        join(folder, 'movielens.zip')
    )



from os import mkdir
from os.path import join, exists

import datatable as dt

from rs_datasets.data_loader import download_dataset
from rs_datasets.generic_dataset import Dataset, safe


class Epinions(Dataset):
    def __init__(self, path: str = None):
        """
        :param path: folder which is used to download dataset to
            if it does not contain dataset files.
            If files are found, load them.
        """
        super().__init__(path)
        folder = join(self.data_folder, 'epinions')
        if not exists(folder):
            self._download(folder)

        self.ratings = dt.fread(
            join(folder, 'ratings_data.txt'),
            columns=['user_id', 'item_id', 'rating']
        ).to_pandas()

        self.trust = dt.fread(
            join(folder, 'trust_data.txt'),
            columns=['source_user_id', 'target_user_id', 'trust_value']
        ).to_pandas()

    @safe
    def _download(self, path):
        self.logger.info('Downloading Epinions dataset...')
        mkdir(path)
        base_url = 'http://www.trustlet.org/datasets/downloaded_epinions/'

        filepath = join(path, 'ratings_data.txt.bz2')
        download_dataset(
            base_url + 'ratings_data.txt.bz2',
            filepath,
            manage_folder=False
        )

        filepath = join(path, 'trust_data.txt.bz2')
        download_dataset(
            base_url + 'trust_data.txt.bz2',
            filepath,
            manage_folder=False
        )



from os import mkdir
from os.path import join, exists

import datatable as dt

from rs_datasets.data_loader import download_url
from rs_datasets.generic_dataset import Dataset, safe

categories = {
    'fashion': 'AMAZON_FASHION.csv',
    'beauty': 'All_Beauty.csv',
    'appliances': 'Appliances.csv',
    'arts': 'Arts_Crafts_and_Sewing.csv',
    'automotive': 'Automotive.csv',
    'books': 'Books.csv',
    'cds': 'CDs_and_Vinyl.csv',
    'phones': 'Cell_Phones_and_Accessories.csv',
    'clothing': 'Clothing_Shoes_and_Jewelry.csv',
    'music': 'Digital_Music.csv',
    'electronics': 'Electronics.csv',
    'cards': 'Gift_Cards.csv',
    'grocery': 'Grocery_and_Gourmet_Food.csv',
    'kitchen': 'Home_and_Kitchen.csv',
    'scientific': 'Industrial_and_Scientific.csv',
    'kindle': 'Kindle_Store.csv',
    'luxury': 'Luxury_Beauty.csv',
    'subscriptions': 'Magazine_Subscriptions.csv',
    'movies': 'Movies_and_TV.csv',
    'musical instruments': 'Musical_Instruments.csv',
    'office': 'Office_Products.csv',
    'garden': 'Patio_Lawn_and_Garden.csv',
    'pet': 'Pet_Supplies.csv',
    'pantry': 'Prime_Pantry.csv',
    'software': 'Software.csv',
    'sports': 'Sports_and_Outdoors.csv',
    'tools': 'Tools_and_Home_Improvement.csv',
    'toys': 'Toys_and_Games.csv',
    'games': 'Video_Games.csv'
}


class Amazon(Dataset):
    def __init__(self, category, path: str = None):
        """
        :param category: one of {'fashion', 'beauty', 'appliances', 'arts', 'automotive',
              'books', 'cds', 'phones', 'clothing', 'music', 'electronics',
              'cards', 'grocery', 'kitchen', 'scientific', 'kindle', 'luxury',
              'subscriptions', 'movies', 'musical instruments', 'office', 'garden',
              'pet', 'pantry', 'software', 'sports', 'tools', 'toys', 'games}
        :param path: folder which is used to download dataset to
            if it does not contain dataset files.
            If files are found, load them.
        """
        super().__init__(path)
        if category not in categories:
            raise ValueError(f'Dataset category must be one of {categories.keys()}')
        folder = join(self.data_folder, 'amazon')
        if not exists(folder):
            mkdir(folder)
        if not exists(join(folder, category + '.csv')):
            self._download(folder, category)
        self.ratings = dt.fread(
            join(folder, category + '.csv'),
            columns=['user_id', 'item_id', 'rating']
        ).to_pandas()


    @safe
    def _download(self, path, category):
        self.logger.info(f'Downloading Amazon {category} ratings...')
        base_url = 'http://deepyeti.ucsd.edu/jianmo/amazon/categoryFilesSmall/'
        url = base_url + categories[category]
        download_url(url, join(path, category + '.csv'))



from os import rename
from os.path import join, exists

import datatable as dt

from rs_datasets.data_loader import download_dataset
from rs_datasets.generic_dataset import Dataset, safe


class DatingAgency(Dataset):
    def __init__(self, path: str = None):
        """
        :param path: folder which is used to download dataset to
            if it does not contain dataset files.
            If files are found, load them.
        """
        super().__init__(path)
        folder = join(self.data_folder, 'dating_agency')
        if not exists(folder):
            self._download(folder)

        self.ratings = dt.fread(
            join(folder, 'ratings.dat'),
            columns=['user_id', 'profile_id', 'rating']
        ).to_pandas()

        self.users = dt.fread(
            join(folder, 'gender.dat'),
            columns=['user_id', 'gender']
        ).to_pandas()

    @safe
    def _download(self, path):
        self.logger.info('Downloading Dating Agency Dataset...')
        url = 'http://www.occamslab.com/petricek/data/libimseti-complete.zip'
        download_dataset(url, join(self.data_folder, 'dating.zip'))
        rename(join(self.data_folder, 'libimseti'), path)



from urllib.request import urlretrieve

from tqdm import tqdm

from rs_datasets.data_loader.archives import extract, rm_if_exists


def download_dataset(
        url: str, destination_path: str, manage_folder: bool = True):
    """
    Download dataset from the internet.

    :param url: from where
    :param destination_path: where to
    :param manage_folder: check if there is root folder in archive:
        if there is one, do not create extra folder,
        if there are just files inside, put them into folder.
        If param is set to `False`, extract "as is".
    :return: None
    """
    download_url(url, destination_path)
    extract(destination_path, manage_folder)
    rm_if_exists(destination_path)


def download_url(url: str, filename: str):
    """
    Download something from link.

    :param url: link
    :param filename: path to save
    :return: None
    """
    with tqdm(unit='B', unit_scale=True) as progress:
        def report(chunk, chunksize, total):
            progress.total = total
            progress.update(chunksize)
        return urlretrieve(url, filename, reporthook=report)
